﻿using HeuristicLab.Algorithms.PushGP.Expressions;
using HeuristicLab.Algorithms.PushGP.Generators;
using HeuristicLab.Algorithms.PushGP.Stack;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace HeuristicLab.Tests.Interpreter.Expressions
{
    [TestClass]
    public class NameExpressionTests : CommonTests<string>
    {

        protected override string TypeName { get { return "NAME"; } }
        protected override IStack<string> Stack { get { return interpreter.NameStack; } }

        [TestMethod]
        [TestProperty("Time", "Short")]
        [TestCategory("ExpressionTest")]
        [TestCategory("NameExpressionTest")]
        public void TestRand()
        {
            interpreter.Interpret(new NameRandExpression());

            Assert.IsTrue(interpreter.NameStack.Count == 1);

            CheckOtherStacksAreEmpty();
        }

        [TestMethod]
        [TestProperty("Time", "Short")]
        [TestCategory("ExpressionTest")]
        [TestCategory("NameExpressionTest")]
        public void TestRandomBound()
        {
            interpreter.CustomExpressions.Add("c1", new CodeNoopExpression());
            interpreter.CustomExpressions.Add("c2", new CodeNoopExpression());
            interpreter.CustomExpressions.Add("c3", new CodeNoopExpression());

            interpreter.Interpret(new NameRandBoundNameExpression());

            Assert.IsTrue(interpreter.CustomExpressions.Keys.Contains(interpreter.NameStack.Top));

            CheckOtherStacksAreEmpty();
        }

        [TestMethod]
        [TestProperty("Time", "Short")]
        [TestCategory("ExpressionTest")]
        [TestCategory("NameExpressionTest")]
        public void TestRandomBoundWithInsufficientArguments()
        {
            TestWithInsufficientArguments("RANDBOUNDNAME");
        }

        [TestMethod]
        [TestProperty("Time", "Short")]
        [TestCategory("ExpressionTest")]
        [TestCategory("NameExpressionTest")]
        public void TestQuote()
        {
            interpreter.Interpret(new NameQuoteExpression());

            Assert.IsTrue(interpreter.IsNameQuoteFlagSet);

            TestStackCounts();
        }

        protected override string[] GetValues(int count)
        {
            var ng = new NameGenerator();
            var values = new string[count];

            for (var i = 0; i < count; i++)
            {
                values[i] = ng.GetNextString();
            }

            return values;
        }

        protected override void CheckOtherStacksAreEmpty()
        {
            TestStackCounts(nameStack: null);
        }
    }
}
